@@ -31,15 +31,17 @@
 #include "mm-daemon-enums-types.h"
 #include "mm-iface-modem.h"
 #include "mm-iface-modem-3gpp.h"
+#include "mm-iface-modem-3gpp-profile-manager.h"
 #include "mm-iface-modem-cdma.h"
 #include "mm-base-bearer.h"
-#include "mm-base-modem-at.h"
 #include "mm-base-modem.h"
 #include "mm-log-object.h"
 #include "mm-modem-helpers.h"
 #include "mm-error-helpers.h"
 #include "mm-bearer-stats.h"
 #include "mm-dispatcher-connection.h"
+#include "mm-auth-provider.h"
+#include "mm-bind.h"
 
 /* We require up to 20s to get a proper IP when using PPP */
 #define BEARER_IP_TIMEOUT_DEFAULT 20
@@ -53,9 +55,11 @@
 #define BEARER_CONNECTION_MONITOR_TIMEOUT          5
 
 static void log_object_iface_init (MMLogObjectInterface *iface);
+static void bind_iface_init (MMBindInterface *iface);
 
 G_DEFINE_TYPE_EXTENDED (MMBaseBearer, mm_base_bearer, MM_GDBUS_TYPE_BEARER_SKELETON, 0,
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_LOG_OBJECT, log_object_iface_init))
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_LOG_OBJECT, log_object_iface_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_BIND, bind_iface_init))
 
 typedef enum {
     CONNECTION_FORBIDDEN_REASON_NONE,
@@ -69,6 +73,7 @@ enum {
     PROP_0,
     PROP_PATH,
     PROP_CONNECTION,
+    PROP_BIND_TO,
     PROP_MODEM,
     PROP_STATUS,
     PROP_CONFIG,
@@ -82,6 +87,13 @@ struct _MMBaseBearerPrivate {
     GDBusConnection *connection;
     guint            dbus_id;
 
+    /* The authorization provider */
+    MMAuthProvider *authp;
+    GCancellable   *authp_cancellable;
+
+    /* The object this bearer is bound to */
+    GObject *bind_to;
+
     /* The modem which owns this BEARER */
     MMBaseModem *modem;
     /* The path where the BEARER object is exported */
@@ -251,6 +263,8 @@ bearer_update_connection_error (MMBaseBearer *self,
     g_autoptr(GVariant) tuple = NULL;
 
     if (connection_error) {
+        g_autoptr(GError) normalized_error = NULL;
+
         /* Never overwrite a connection error if it's already set */
         tuple = mm_gdbus_bearer_dup_connection_error (MM_GDBUS_BEARER (self));
         if (tuple)
@@ -259,42 +273,9 @@ bearer_update_connection_error (MMBaseBearer *self,
         /*
          * Limit the type of errors we can expose in the interface;
          * e.g. we don't want QMI or MBIM specific errors reported.
-         *
-         * G_IO_ERROR_CANCELLED is an exception, because we map it to
-         * MM_CORE_ERROR_CANCELLED implicitly when building the DBus error name.
          */
-        if ((connection_error->domain != MM_CORE_ERROR) &&
-            (connection_error->domain != MM_MOBILE_EQUIPMENT_ERROR) &&
-            (connection_error->domain != MM_CONNECTION_ERROR) &&
-            (connection_error->domain != MM_SERIAL_ERROR) &&
-            (connection_error->domain != MM_CDMA_ACTIVATION_ERROR) &&
-            (!g_error_matches (connection_error, G_IO_ERROR, G_IO_ERROR_CANCELLED))) {
-            g_autoptr(GError) default_connection_error = NULL;
-
-#if defined WITH_QMI
-            if (connection_error->domain == QMI_CORE_ERROR)
-                mm_obj_dbg (self, "cannot set QMI core error as connection error: %s", connection_error->message);
-            else if (connection_error->domain == QMI_PROTOCOL_ERROR)
-                mm_obj_dbg (self, "cannot set QMI protocol error as connection error: %s", connection_error->message);
-            else
-#endif
-#if defined WITH_MBIM
-            if (connection_error->domain == MBIM_CORE_ERROR)
-                mm_obj_dbg (self, "cannot set MBIM core error as connection error: %s", connection_error->message);
-            else if (connection_error->domain == MBIM_PROTOCOL_ERROR)
-                mm_obj_dbg (self, "cannot set MBIM protocol error as connection error: %s", connection_error->message);
-            else if (connection_error->domain == MBIM_STATUS_ERROR)
-                mm_obj_dbg (self, "cannot set MBIM status error as connection error: %s", connection_error->message);
-            else
-#endif
-                mm_obj_dbg (self, "cannot set unhandled domain error as connection error: %s", connection_error->message);
-
-            default_connection_error = g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
-                                                    MM_MOBILE_EQUIPMENT_ERROR_UNKNOWN,
-                                                    "%s", connection_error->message);
-            tuple = mm_common_error_to_tuple (default_connection_error);
-        } else
-            tuple = mm_common_error_to_tuple (connection_error);
+        normalized_error = mm_normalize_error (connection_error);
+        tuple = mm_common_error_to_tuple (normalized_error);
     }
     mm_gdbus_bearer_set_connection_error (MM_GDBUS_BEARER (self), tuple);
 }
@@ -491,7 +472,7 @@ dispatcher_connection_run_ready (MMDispatcherConnection *dispatcher,
 
 static void
 bearer_run_dispatcher_scripts (MMBaseBearer *self,
-                               gboolean      connected)
+                               MMDispatcherConnectionEvent event)
 {
     MMDispatcherConnection *dispatcher;
     const gchar *interface;
@@ -505,7 +486,7 @@ bearer_run_dispatcher_scripts (MMBaseBearer *self,
                                   g_dbus_object_get_object_path (G_DBUS_OBJECT (self->priv->modem)),
                                   self->priv->path,
                                   interface,
-                                  connected,
+                                  event,
                                   NULL, /* cancellable */
                                   (GAsyncReadyCallback)dispatcher_connection_run_ready,
                                   g_object_ref (self));
@@ -519,6 +500,7 @@ bearer_reset_interface_status (MMBaseBearer *self)
     mm_gdbus_bearer_set_profile_id (MM_GDBUS_BEARER (self), MM_3GPP_PROFILE_ID_UNKNOWN);
     mm_gdbus_bearer_set_multiplexed (MM_GDBUS_BEARER (self), FALSE);
     mm_gdbus_bearer_set_connected (MM_GDBUS_BEARER (self), FALSE);
+    mm_gdbus_bearer_set_disconnect_request (MM_GDBUS_BEARER (self), FALSE);
     mm_gdbus_bearer_set_suspended (MM_GDBUS_BEARER (self), FALSE);
     mm_gdbus_bearer_set_interface (MM_GDBUS_BEARER (self), NULL);
     mm_gdbus_bearer_set_ip4_config (
@@ -549,8 +531,8 @@ bearer_update_status (MMBaseBearer *self,
     if (self->priv->status == MM_BEARER_STATUS_DISCONNECTED) {
         g_autoptr(GString) report = NULL;
 
-        /* Report disconnection via dispatcher scripts, before reseting the interface */
-        bearer_run_dispatcher_scripts (self, FALSE);
+        /* Report disconnection via dispatcher scripts, before resetting the interface */
+        bearer_run_dispatcher_scripts (self, MM_DISPATCHER_CONNECTION_EVENT_DISCONNECTED);
 
         bearer_reset_interface_status (self);
         /* Cleanup flag to ignore disconnection reports */
@@ -618,7 +600,7 @@ bearer_update_status_connected (MMBaseBearer     *self,
     connection_monitor_start (self);
 
     /* Run dispatcher scripts */
-    bearer_run_dispatcher_scripts (self, TRUE);
+    bearer_run_dispatcher_scripts (self, MM_DISPATCHER_CONNECTION_EVENT_CONNECTED);
 }
 
 /*****************************************************************************/
@@ -1005,10 +987,17 @@ connect_ready (MMBaseBearer *self,
         mm_obj_warn (self, "connection attempt #%u failed: %s",
                      mm_bearer_stats_get_attempts (self->priv->stats),
                      error->message);
+        /* process profile manager updates right away on error */
+        if (mm_iface_modem_is_3gpp (MM_IFACE_MODEM (self->priv->modem)))
+            mm_iface_modem_3gpp_profile_manager_update_ignore_stop (MM_IFACE_MODEM_3GPP_PROFILE_MANAGER (self->priv->modem));
         connect_failed (self, task, error);
         return;
     }
 
+    /* delay processing profile manager updates on success */
+    if (mm_iface_modem_is_3gpp (MM_IFACE_MODEM (self->priv->modem)))
+        mm_iface_modem_3gpp_profile_manager_update_ignore_stop_delayed (MM_IFACE_MODEM_3GPP_PROFILE_MANAGER (self->priv->modem));
+
     /* Handle cancellations detected after successful connection */
     if (connect_check_cancel (self, task))
         return;
@@ -1124,6 +1113,10 @@ mm_base_bearer_connect (MMBaseBearer *self,
     /* Clear previous connection error, if any */
     bearer_update_connection_error (self, NULL);
 
+    /* The connect request may imply a profile update internally, so ignore it */
+    if (mm_iface_modem_is_3gpp (MM_IFACE_MODEM (self->priv->modem)))
+        mm_iface_modem_3gpp_profile_manager_update_ignore_start (MM_IFACE_MODEM_3GPP_PROFILE_MANAGER (self->priv->modem));
+
     /* Connecting! */
     mm_obj_dbg (self, "connecting...");
     self->priv->connect_cancellable = g_cancellable_new ();
@@ -1137,7 +1130,6 @@ mm_base_bearer_connect (MMBaseBearer *self,
 
 typedef struct {
     MMBaseBearer *self;
-    MMBaseModem *modem;
     GDBusMethodInvocation *invocation;
 } HandleConnectContext;
 
@@ -1145,7 +1137,6 @@ static void
 handle_connect_context_free (HandleConnectContext *ctx)
 {
     g_object_unref (ctx->invocation);
-    g_object_unref (ctx->modem);
     g_object_unref (ctx->self);
     g_free (ctx);
 }
@@ -1158,7 +1149,7 @@ handle_connect_ready (MMBaseBearer *self,
     GError *error = NULL;
 
     if (!mm_base_bearer_connect_finish (self, res, &error))
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
     else
         mm_gdbus_bearer_complete_connect (MM_GDBUS_BEARER (self), ctx->invocation);
 
@@ -1166,18 +1157,19 @@ handle_connect_ready (MMBaseBearer *self,
 }
 
 static void
-handle_connect_auth_ready (MMBaseModem *modem,
+handle_connect_auth_ready (MMAuthProvider *authp,
                            GAsyncResult *res,
                            HandleConnectContext *ctx)
 {
     GError *error = NULL;
 
-    if (!mm_base_modem_authorize_finish (modem, res, &error)) {
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+    if (!mm_auth_provider_authorize_finish (authp, res, &error)) {
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
         handle_connect_context_free (ctx);
         return;
     }
 
+    mm_obj_info (ctx->self, "processing user request to connect...");
     mm_base_bearer_connect (ctx->self,
                             (GAsyncReadyCallback)handle_connect_ready,
                             ctx);
@@ -1192,17 +1184,13 @@ handle_connect (MMBaseBearer *self,
     ctx = g_new0 (HandleConnectContext, 1);
     ctx->self = g_object_ref (self);
     ctx->invocation = g_object_ref (invocation);
-    g_object_get (self,
-                  MM_BASE_BEARER_MODEM, &ctx->modem,
-                  NULL);
 
-    mm_obj_dbg (self, "user request to connect");
-
-    mm_base_modem_authorize (ctx->modem,
-                             invocation,
-                             MM_AUTHORIZATION_DEVICE_CONTROL,
-                             (GAsyncReadyCallback)handle_connect_auth_ready,
-                             ctx);
+    mm_auth_provider_authorize (self->priv->authp,
+                                invocation,
+                                MM_AUTHORIZATION_DEVICE_CONTROL,
+                                self->priv->authp_cancellable,
+                                (GAsyncReadyCallback)handle_connect_auth_ready,
+                                ctx);
     return TRUE;
 }
 
@@ -1297,8 +1285,6 @@ mm_base_bearer_disconnect (MMBaseBearer *self,
         return;
     }
 
-    mm_obj_dbg (self, "disconnecting...");
-
     /* If currently connecting, try to cancel that operation, and wait to get
      * disconnected. */
     if (self->priv->status == MM_BEARER_STATUS_CONNECTING) {
@@ -1329,7 +1315,6 @@ mm_base_bearer_disconnect (MMBaseBearer *self,
 
 typedef struct {
     MMBaseBearer *self;
-    MMBaseModem *modem;
     GDBusMethodInvocation *invocation;
 } HandleDisconnectContext;
 
@@ -1337,7 +1322,6 @@ static void
 handle_disconnect_context_free (HandleDisconnectContext *ctx)
 {
     g_object_unref (ctx->invocation);
-    g_object_unref (ctx->modem);
     g_object_unref (ctx->self);
     g_free (ctx);
 }
@@ -1350,7 +1334,7 @@ handle_disconnect_ready (MMBaseBearer *self,
     GError *error = NULL;
 
     if (!mm_base_bearer_disconnect_finish (self, res, &error))
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
     else
         mm_gdbus_bearer_complete_disconnect (MM_GDBUS_BEARER (self), ctx->invocation);
 
@@ -1358,18 +1342,19 @@ handle_disconnect_ready (MMBaseBearer *self,
 }
 
 static void
-handle_disconnect_auth_ready (MMBaseModem *modem,
+handle_disconnect_auth_ready (MMAuthProvider *authp,
                               GAsyncResult *res,
                               HandleDisconnectContext *ctx)
 {
     GError *error = NULL;
 
-    if (!mm_base_modem_authorize_finish (modem, res, &error)) {
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+    if (!mm_auth_provider_authorize_finish (authp, res, &error)) {
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
         handle_disconnect_context_free (ctx);
         return;
     }
 
+    mm_obj_info (ctx->self, "processing user request to disconnect...");
     mm_base_bearer_disconnect (ctx->self,
                                (GAsyncReadyCallback)handle_disconnect_ready,
                                ctx);
@@ -1384,17 +1369,13 @@ handle_disconnect (MMBaseBearer *self,
     ctx = g_new0 (HandleDisconnectContext, 1);
     ctx->self = g_object_ref (self);
     ctx->invocation = g_object_ref (invocation);
-    g_object_get (self,
-                  MM_BASE_BEARER_MODEM, &ctx->modem,
-                  NULL);
 
-    mm_obj_dbg (self, "user request to disconnect");
-
-    mm_base_modem_authorize (ctx->modem,
-                             invocation,
-                             MM_AUTHORIZATION_DEVICE_CONTROL,
-                             (GAsyncReadyCallback)handle_disconnect_auth_ready,
-                             ctx);
+    mm_auth_provider_authorize (self->priv->authp,
+                                invocation,
+                                MM_AUTHORIZATION_DEVICE_CONTROL,
+                                self->priv->authp_cancellable,
+                                (GAsyncReadyCallback)handle_disconnect_auth_ready,
+                                ctx);
     return TRUE;
 }
 
@@ -1511,11 +1492,14 @@ mm_base_bearer_disconnect_force (MMBaseBearer *self)
         return;
 
     if (self->priv->ignore_disconnection_reports) {
-        mm_obj_dbg (self, "disconnection should be forced but it's explicitly ignored");
+        mm_obj_msg (self, "disconnection should be forced, but we can't. Request disconnection instead.");
+        mm_gdbus_bearer_set_disconnect_request (MM_GDBUS_BEARER(self), TRUE);
+        bearer_run_dispatcher_scripts (self,
+                                       MM_DISPATCHER_CONNECTION_EVENT_DISCONNECT_REQUEST);
         return;
     }
 
-    mm_obj_dbg (self, "forcing disconnection");
+    mm_obj_msg (self, "forcing disconnection");
 
     /* If currently connecting, try to cancel that operation. */
     if (self->priv->status == MM_BEARER_STATUS_CONNECTING) {
@@ -1770,17 +1754,15 @@ set_property (GObject *object,
         else if (self->priv->path)
             base_bearer_dbus_export (self);
         break;
+    case PROP_BIND_TO:
+        g_clear_object (&self->priv->bind_to);
+        self->priv->bind_to = g_value_dup_object (value);
+        mm_bind_to (MM_BIND (self), MM_BASE_BEARER_CONNECTION, self->priv->bind_to);
+        break;
     case PROP_MODEM:
         g_clear_object (&self->priv->modem);
         self->priv->modem = g_value_dup_object (value);
         if (self->priv->modem) {
-            /* Set owner ID */
-            mm_log_object_set_owner_id (MM_LOG_OBJECT (self), mm_log_object_get_id (MM_LOG_OBJECT (self->priv->modem)));
-            /* Bind the modem's connection (which is set when it is exported,
-             * and unset when unexported) to the BEARER's connection */
-            g_object_bind_property (self->priv->modem, MM_BASE_MODEM_CONNECTION,
-                                    self, MM_BASE_BEARER_CONNECTION,
-                                    G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
             if (self->priv->config) {
                 /* Listen to 3GPP/CDMA registration state changes. We need both
                  * 'config' and 'modem' set. */
@@ -1830,6 +1812,9 @@ get_property (GObject *object,
     case PROP_CONNECTION:
         g_value_set_object (value, self->priv->connection);
         break;
+    case PROP_BIND_TO:
+        g_value_set_object (value, self->priv->bind_to);
+        break;
     case PROP_MODEM:
         g_value_set_object (value, self->priv->modem);
         break;
@@ -1858,6 +1843,10 @@ mm_base_bearer_init (MMBaseBearer *self)
     /* Each bearer is given a unique id to build its own DBus path */
     self->priv->dbus_id = id++;
 
+    /* Setup authorization provider */
+    self->priv->authp = mm_auth_provider_get ();
+    self->priv->authp_cancellable = g_cancellable_new ();
+
     self->priv->status = MM_BEARER_STATUS_DISCONNECTED;
     self->priv->reason_3gpp = CONNECTION_FORBIDDEN_REASON_NONE;
     self->priv->reason_cdma = CONNECTION_FORBIDDEN_REASON_NONE;
@@ -1908,7 +1897,10 @@ dispose (GObject *object)
     reset_deferred_unregistration (self);
 
     g_clear_object (&self->priv->modem);
+    g_clear_object (&self->priv->bind_to);
     g_clear_object (&self->priv->config);
+    g_cancellable_cancel (self->priv->authp_cancellable);
+    g_clear_object (&self->priv->authp_cancellable);
 
     G_OBJECT_CLASS (mm_base_bearer_parent_class)->dispose (object);
 }
@@ -1919,6 +1911,11 @@ log_object_iface_init (MMLogObjectInterface *iface)
     iface->build_id = log_object_build_id;
 }
 
+static void
+bind_iface_init (MMBindInterface *iface)
+{
+}
+
 static void
 mm_base_bearer_class_init (MMBaseBearerClass *klass)
 {
@@ -1950,6 +1947,8 @@ mm_base_bearer_class_init (MMBaseBearerClass *klass)
                              G_PARAM_READWRITE);
     g_object_class_install_property (object_class, PROP_PATH, properties[PROP_PATH]);
 
+    g_object_class_override_property (object_class, PROP_BIND_TO, MM_BIND_TO);
+
     properties[PROP_MODEM] =
         g_param_spec_object (MM_BASE_BEARER_MODEM,
                              "Modem",
