@@ -17,8 +17,6 @@
  * Copyright (C) 2011 - 2016 Aleksander Morgado <aleksander@aleksander.es>
  */
 
-#include "gio/gio.h"
-#include "glib.h"
 #include <config.h>
 
 #include <string.h>
@@ -872,42 +870,48 @@ mm_base_manager_start (MMBaseManager *self,
 /*****************************************************************************/
 
 typedef enum {
-    DEVICE_CLEANUP_STEP_FIRST = 0,
-    DEVICE_CLEANUP_STEP_DISABLE,
-    DEVICE_CLEANUP_STEP_LOW_POWER,
-    DEVICE_CLEANUP_STEP_REMOVE,
-    DEVICE_CLEANUP_STEP_TERSE,
-    DEVICE_CLEANUP_STEP_LAST,
-} DeviceCleanupStep;
+    DEVICE_SHUTDOWN_STEP_FIRST = 0,
+    DEVICE_SHUTDOWN_STEP_DISABLE,
+    DEVICE_SHUTDOWN_STEP_LOW_POWER,
+    DEVICE_SHUTDOWN_STEP_REMOVE,
+    DEVICE_SHUTDOWN_STEP_LAST,
+} DeviceShutdownStep;
 
 typedef struct {
-    MMBaseModem               *modem;
-    MMBaseManagerCleanupFlags  flags;
-    DeviceCleanupStep          step;
-} DeviceCleanupContext;
+    MMBaseModem *modem;
+    gboolean     disable;
+    gboolean     low_power;
+    gboolean     remove;
 
-static void device_cleanup_step (GTask *task);
+    DeviceShutdownStep step;
+} DeviceShutdownContext;
 
-static DeviceCleanupContext *
-device_cleanup_context_new (MMBaseModem *modem,
-                            MMBaseManagerCleanupFlags  flags)
+static void device_shutdown_step (GTask *task);
+
+static DeviceShutdownContext *
+device_shutdown_context_new (MMBaseModem *modem,
+                             gboolean     disable,
+                             gboolean     low_power,
+                             gboolean     remove)
 {
-    DeviceCleanupContext *ctx;
+    DeviceShutdownContext *ctx;
 
-    ctx = g_slice_new0 (DeviceCleanupContext);
+    ctx = g_slice_new0 (DeviceShutdownContext);
     if (modem)
         ctx->modem = g_object_ref (modem);
-    ctx->step = DEVICE_CLEANUP_STEP_FIRST;
-    ctx->flags = flags;
+    ctx->step = DEVICE_SHUTDOWN_STEP_FIRST;
+    ctx->disable = disable;
+    ctx->low_power = low_power;
+    ctx->remove = remove;
 
     return ctx;
 }
 
 static void
-device_cleanup_context_free (DeviceCleanupContext *ctx)
+device_shutdown_context_free (DeviceShutdownContext *ctx)
 {
     g_clear_object (&ctx->modem);
-    g_slice_free (DeviceCleanupContext, ctx);
+    g_slice_free (DeviceShutdownContext, ctx);
 }
 
 static void
@@ -915,7 +919,7 @@ modem_remove_ready (MMDevice     *device,
                     GAsyncResult *res,
                     GTask        *task)
 {
-    DeviceCleanupContext *ctx;
+    DeviceShutdownContext *ctx;
     g_autoptr(GError)      error = NULL;
 
     ctx = g_task_get_task_data (task);
@@ -924,15 +928,15 @@ modem_remove_ready (MMDevice     *device,
         mm_obj_warn (device, "removing modem failed: %s", error->message);
 
     ctx->step++;
-    device_cleanup_step (task);
+    device_shutdown_step (task);
 }
 
 static void
-cleanup_low_power_ready (MMIfaceModem *modem,
-                         GAsyncResult *res,
-                         GTask        *task)
+shutdown_low_power_ready (MMIfaceModem *modem,
+                          GAsyncResult *res,
+                          GTask        *task)
 {
-    DeviceCleanupContext *ctx;
+    DeviceShutdownContext *ctx;
     g_autoptr(GError)      error = NULL;
 
     ctx = g_task_get_task_data (task);
@@ -941,15 +945,15 @@ cleanup_low_power_ready (MMIfaceModem *modem,
         mm_obj_info (modem, "changing to low power state failed: %s", error->message);
 
     ctx->step++;
-    device_cleanup_step (task);
+    device_shutdown_step (task);
 }
 
 static void
-cleanup_disable_ready (MMBaseModem  *modem,
-                       GAsyncResult *res,
-                       GTask        *task)
+shutdown_disable_ready (MMBaseModem  *modem,
+                        GAsyncResult *res,
+                        GTask        *task)
 {
-    DeviceCleanupContext *ctx;
+    DeviceShutdownContext *ctx;
     g_autoptr(GError)      error = NULL;
 
     ctx = g_task_get_task_data (task);
@@ -962,83 +966,57 @@ cleanup_disable_ready (MMBaseModem  *modem,
         /* If disabling failed there's no point in trying to low-power the
          * modem; just remove it.
          */
-        ctx->step = DEVICE_CLEANUP_STEP_REMOVE;
+        ctx->step = DEVICE_SHUTDOWN_STEP_REMOVE;
     }
 
-    device_cleanup_step (task);
-}
-
-#if defined WITH_SUSPEND_RESUME
-static void
-cleanup_terse_ready (MMBaseModem  *modem,
-                     GAsyncResult *res,
-                     GTask        *task)
-{
-    DeviceCleanupContext *ctx;
-    g_autoptr(GError)     error = NULL;
-
-    ctx = g_task_get_task_data (task);
-    if (!mm_base_modem_terse_finish (modem, res, &error))
-        mm_obj_info (modem, "setting terse failed: %s", error->message);
-
-    ctx->step++;
-
-    device_cleanup_step (task);
-}
-#endif
-
-static gboolean
-has_cleanup_flag (MMBaseManagerCleanupFlags flags,
-                  MMBaseManagerCleanupFlags expected)
-{
-    return flags & expected;
+    device_shutdown_step (task);
 }
 
 static void
-device_cleanup_step (GTask *task)
+device_shutdown_step (GTask *task)
 {
-    DeviceCleanupContext *ctx;
+    DeviceShutdownContext *ctx;
     MMDevice              *device;
 
     ctx = g_task_get_task_data (task);
     device = g_task_get_source_object (task);
 
     switch (ctx->step) {
-    case DEVICE_CLEANUP_STEP_FIRST:
+    case DEVICE_SHUTDOWN_STEP_FIRST:
         ctx->step++;
         /* fall through */
 
-    case DEVICE_CLEANUP_STEP_DISABLE:
-        if (ctx->modem && has_cleanup_flag (ctx->flags, MM_BASE_MANAGER_CLEANUP_DISABLE)) {
+    case DEVICE_SHUTDOWN_STEP_DISABLE:
+        if (ctx->disable && ctx->modem) {
             mm_obj_dbg (device,
                         "disabling modem%d",
                         mm_base_modem_get_dbus_id (ctx->modem));
             mm_base_modem_disable (ctx->modem,
-                                   MM_BASE_MODEM_OPERATION_LOCK_REQUIRED,
-                                   MM_BASE_MODEM_OPERATION_PRIORITY_OVERRIDE,
-                                   (GAsyncReadyCallback)cleanup_disable_ready,
+                                   MM_OPERATION_LOCK_REQUIRED,
+                                   MM_OPERATION_PRIORITY_OVERRIDE,
+                                   (GAsyncReadyCallback)shutdown_disable_ready,
                                    task);
             return;
         }
         ctx->step++;
         /* fall through */
 
-    case DEVICE_CLEANUP_STEP_LOW_POWER:
-        if (ctx->modem && has_cleanup_flag (ctx->flags, MM_BASE_MANAGER_CLEANUP_LOW_POWER)) {
+    case DEVICE_SHUTDOWN_STEP_LOW_POWER:
+        if (ctx->low_power && ctx->modem) {
             mm_obj_dbg (device,
                         "setting modem%d to low-power state",
                         mm_base_modem_get_dbus_id (ctx->modem));
             mm_iface_modem_set_power_state (MM_IFACE_MODEM (ctx->modem),
                                             MM_MODEM_POWER_STATE_LOW,
-                                            (GAsyncReadyCallback)cleanup_low_power_ready,
+                                            (GAsyncReadyCallback)shutdown_low_power_ready,
                                             task);
             return;
         }
         ctx->step++;
         /* fall through */
 
-    case DEVICE_CLEANUP_STEP_REMOVE:
-        if (ctx->modem && has_cleanup_flag (ctx->flags, MM_BASE_MANAGER_CLEANUP_REMOVE)) {
+    case DEVICE_SHUTDOWN_STEP_REMOVE:
+        if (ctx->remove && ctx->modem) {
             mm_obj_dbg (device,
                         "removing modem%d",
                         mm_base_modem_get_dbus_id (ctx->modem));
@@ -1050,26 +1028,8 @@ device_cleanup_step (GTask *task)
         ctx->step++;
         /* fall through */
 
-    case DEVICE_CLEANUP_STEP_TERSE:
-
-#if defined WITH_SUSPEND_RESUME
-        if (ctx->modem && has_cleanup_flag (ctx->flags, MM_BASE_MANAGER_CLEANUP_TERSE)) {
-            mm_obj_dbg(device,
-                    "settting terse on modem%d",
-                    mm_base_modem_get_dbus_id (ctx->modem));
-            mm_base_modem_terse (ctx->modem,
-                                 MM_BASE_MODEM_OPERATION_LOCK_REQUIRED,
-                                 (GAsyncReadyCallback)cleanup_terse_ready,
-                                 task);
-            return;
-        }
-#endif
-
-        ctx->step++;
-        /* fall through */
-
-    case DEVICE_CLEANUP_STEP_LAST:
-        mm_obj_dbg (device, "device cleanup complete");
+    case DEVICE_SHUTDOWN_STEP_LAST:
+        mm_obj_dbg (device, "device shutdown complete");
         g_task_return_boolean (task, TRUE);
         g_object_unref (task);
         return;
@@ -1086,25 +1046,25 @@ typedef struct {
     GSList         *tasks;
     GSList         *devices_to_remove;
     MMSleepContext *sleep_ctx;
-} ManagerCleanupContext;
+} ManagerShutdownContext;
 
-static ManagerCleanupContext *
-manager_cleanup_context_new (MMBaseManager  *self,
-                             MMSleepContext *sleep_ctx)
+static ManagerShutdownContext *
+manager_shutdown_context_new (MMBaseManager  *self,
+                              MMSleepContext *sleep_ctx)
 {
-    ManagerCleanupContext *ctx;
+    ManagerShutdownContext *ctx;
 
-    ctx = g_slice_new0 (ManagerCleanupContext);
+    ctx = g_slice_new0 (ManagerShutdownContext);
     ctx->self = g_object_ref (self);
     ctx->sleep_ctx = g_object_ref (sleep_ctx);
     return ctx;
 }
 
 static void
-manager_cleanup_context_maybe_complete (ManagerCleanupContext *ctx,
-                                        GTask                  *task,
-                                        MMDevice               *device,
-                                        gboolean                remove)
+manager_shutdown_context_maybe_complete (ManagerShutdownContext *ctx,
+                                         GTask                  *task,
+                                         MMDevice               *device,
+                                         gboolean                remove)
 {
     if (task)
         ctx->tasks = g_slist_remove (ctx->tasks, task);
@@ -1125,49 +1085,50 @@ manager_cleanup_context_maybe_complete (ManagerCleanupContext *ctx,
             g_hash_table_remove (ctx->self->priv->devices, mm_device_get_uid (device));
         g_slist_free (ctx->devices_to_remove);
 
-        mm_obj_dbg (ctx->self, "cleanup complete");
+        mm_obj_dbg (ctx->self, "shutdown complete");
         mm_sleep_context_complete (ctx->sleep_ctx, NULL);
 
         g_object_unref (ctx->sleep_ctx);
         g_object_unref (ctx->self);
-        g_slice_free (ManagerCleanupContext, ctx);
+        g_slice_free (ManagerShutdownContext, ctx);
     }
 }
 
 static void
-device_cleanup_ready (MMDevice               *device,
-                      GAsyncResult           *res,
-                      ManagerCleanupContext *ctx)
+device_shutdown_ready (MMDevice               *device,
+                       GAsyncResult           *res,
+                       ManagerShutdownContext *ctx)
 {
     GTask                 *task;
-    DeviceCleanupContext *dctx;
+    DeviceShutdownContext *dctx;
     g_autoptr(GError)      error = NULL;
-    gboolean               remove;
 
     task = G_TASK (res);
 
     if (!g_task_propagate_boolean (task, &error)) {
         mm_obj_info (ctx->self,
-                     "failed to cleanup device %s: %s",
+                     "failed to shutdown device %s: %s",
                      mm_device_get_uid (device),
                      error->message);
     }
 
     dctx = g_task_get_task_data (task);
-    remove = has_cleanup_flag (dctx->flags, MM_BASE_MANAGER_CLEANUP_REMOVE);
-    manager_cleanup_context_maybe_complete (ctx, task, device, remove);
+    manager_shutdown_context_maybe_complete (ctx,
+                                             task,
+                                             device,
+                                             dctx->remove);
 }
 
 void
-mm_base_manager_cleanup (MMBaseManager             *self,
-                         MMBaseManagerCleanupFlags  flags,
-                         MMSleepContext            *sleep_ctx)
+mm_base_manager_shutdown (MMBaseManager  *self,
+                          gboolean        disable,
+                          gboolean        low_power,
+                          gboolean        remove,
+                          MMSleepContext *sleep_ctx)
 {
-    ManagerCleanupContext *ctx;
+    ManagerShutdownContext *ctx;
     GHashTableIter          iter;
     MMDevice               *device;
-    gboolean                disable, remove, low_power, terse;
-    guint                   active_bearers = 0;
 
     g_return_if_fail (self != NULL);
     g_return_if_fail (MM_IS_BASE_MANAGER (self));
@@ -1175,60 +1136,38 @@ mm_base_manager_cleanup (MMBaseManager             *self,
     /* Cancel all ongoing auth requests */
     g_cancellable_cancel (self->priv->authp_cancellable);
 
-    disable = has_cleanup_flag (flags, MM_BASE_MANAGER_CLEANUP_DISABLE);
-    low_power = has_cleanup_flag (flags, MM_BASE_MANAGER_CLEANUP_LOW_POWER);
-    remove = has_cleanup_flag (flags, MM_BASE_MANAGER_CLEANUP_REMOVE);
-    terse = has_cleanup_flag (flags, MM_BASE_MANAGER_CLEANUP_TERSE);
     mm_obj_dbg (self,
-                "manager shutting down... (%s%s%s%s%s%s%s)",
+                "manager shutting down... (%s%s%s%s%s)",
                 disable ? "disable" : "",
-                disable && (low_power || remove || terse) ? "," : "",
+                disable && (low_power || remove) ? "," : "",
                 low_power ? "low-power" : "",
-                low_power && (remove || terse) ? "," : "",
-                remove ? "remove" : "",
-                (remove && terse) ? "," : "",
-                terse ? "terse" : "");
+                (low_power && remove) ? "," : "",
+                remove ? "remove" : "");
 
-    ctx = manager_cleanup_context_new (self, sleep_ctx);
+    ctx = manager_shutdown_context_new (self, sleep_ctx);
 
     g_hash_table_iter_init (&iter, self->priv->devices);
     while (g_hash_table_iter_next (&iter, NULL, (gpointer) &device)) {
-        DeviceCleanupContext *dctx;
-        GTask                *task;
-        MMBaseModem          *modem;
-
-        modem = mm_device_peek_modem (device);
-        if (modem) {
-            guint count = 0;
-
-            mm_iface_modem_count_bearers (MM_IFACE_MODEM (modem),
-                                          MM_IFACE_MODEM_COUNT_BEARERS_FLAG_ACTIVE,
-                                          &count,
-                                          NULL,
-                                          NULL);
-            active_bearers += count;
-        }
+        DeviceShutdownContext *dctx;
+        GTask                 *task;
 
         task = g_task_new (device,
                            NULL,
-                           (GAsyncReadyCallback)device_cleanup_ready,
+                           (GAsyncReadyCallback)device_shutdown_ready,
                            ctx);
         ctx->tasks = g_slist_append (ctx->tasks, task);
 
-        dctx = device_cleanup_context_new (modem, flags);
-        g_task_set_task_data (task, dctx, (GDestroyNotify)device_cleanup_context_free);
+        dctx = device_shutdown_context_new (mm_device_peek_modem (device),
+                                            disable,
+                                            low_power,
+                                            remove);
+        g_task_set_task_data (task, dctx, (GDestroyNotify)device_shutdown_context_free);
 
-        device_cleanup_step (task);
+        device_shutdown_step (task);
     }
 
-    /* Increase cleanup timeout for each connected bearer; disconnecting a
-     * bearer can take many seconds.
-     */
-    if (active_bearers)
-        mm_sleep_context_timeout_backoff (sleep_ctx, active_bearers * 20);
-
-    /* Complete cleanup if there were no devices */
-    manager_cleanup_context_maybe_complete (ctx, NULL, NULL, FALSE);
+    /* Complete shutdown if there were no devices */
+    manager_shutdown_context_maybe_complete (ctx, NULL, NULL, FALSE);
 }
 
 guint32
@@ -1289,7 +1228,7 @@ mm_base_manager_sync (MMBaseManager *self)
         /* We just want to start the synchronization, we don't need the result */
         if (modem) {
             mm_base_modem_sync (modem,
-                                MM_BASE_MODEM_OPERATION_LOCK_REQUIRED,
+                                MM_OPERATION_LOCK_REQUIRED,
                                 (GAsyncReadyCallback)base_modem_sync_ready,
                                 NULL);
         }
