@@ -39,11 +39,16 @@
 #include "mm-daemon-enums-types.h"
 #include "mm-serial-parsers.h"
 #include "mm-modem-helpers.h"
+#include "mm-bind.h"
 
 static void log_object_iface_init (MMLogObjectInterface *iface);
+static void auth_iface_init (MMIfaceAuthInterface *iface);
+static void op_lock_iface_init (MMIfaceOpLockInterface *iface);
 
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (MMBaseModem, mm_base_modem, MM_GDBUS_TYPE_OBJECT_SKELETON,
-                                  G_IMPLEMENT_INTERFACE (MM_TYPE_LOG_OBJECT, log_object_iface_init))
+                                  G_IMPLEMENT_INTERFACE (MM_TYPE_LOG_OBJECT, log_object_iface_init)
+                                  G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_AUTH, auth_iface_init)
+                                  G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_OP_LOCK, op_lock_iface_init))
 
 /* If we get 10 consecutive timeouts in a serial port, we consider the modem
  * invalid and we request re-probing. */
@@ -150,6 +155,16 @@ struct _MMBaseModemPrivate {
     gboolean  scheduled_operations_forbidden_forever;
 };
 
+static void   mm_base_modem_operation_lock        (MMBaseModem          *self,
+                                                   MMOperationPriority   priority,
+                                                   const gchar          *description,
+                                                   GAsyncReadyCallback   callback,
+                                                   gpointer              user_data);
+static gssize mm_base_modem_operation_lock_finish (MMBaseModem          *self,
+                                                   GAsyncResult         *res,
+                                                   GError              **error);
+
+
 guint
 mm_base_modem_get_dbus_id (MMBaseModem *self)
 {
@@ -192,11 +207,13 @@ port_timed_out_cb (MMPort       *port,
 
 static MMPort *
 base_modem_create_ignored_port (MMBaseModem *self,
+                                MMPortType   ptype,
                                 const gchar *name)
 {
     return MM_PORT (g_object_new (MM_TYPE_PORT,
                                   MM_PORT_DEVICE, name,
-                                  MM_PORT_TYPE,   MM_PORT_TYPE_IGNORED,
+                                  MM_PORT_GROUP,  MM_PORT_GROUP_IGNORED,
+                                  MM_PORT_TYPE,   ptype,
                                   NULL));
 }
 
@@ -328,6 +345,7 @@ base_modem_create_wwan_port (MMBaseModem *self,
         return MM_PORT (g_object_new (MM_TYPE_PORT,
                                       MM_PORT_DEVICE, name,
                                       MM_PORT_SUBSYS, MM_PORT_SUBSYS_WWAN,
+                                      MM_PORT_GROUP, MM_PORT_GROUP_USED,
                                       MM_PORT_TYPE, MM_PORT_TYPE_XMMRPC,
                                       NULL));
 
@@ -345,6 +363,7 @@ static MMPort *
 base_modem_internal_grab_port (MMBaseModem         *self,
                                MMKernelDevice      *kernel_device,
                                gboolean             link_port,
+                               MMPortGroup          pgroup,
                                MMPortType           ptype,
                                MMPortSerialAtFlag   at_pflags,
                                GError             **error)
@@ -379,8 +398,8 @@ base_modem_internal_grab_port (MMBaseModem         *self,
 
     /* Explicitly ignored ports, grab them but explicitly flag them as ignored
      * right away, all the same way (i.e. regardless of subsystem). */
-    if (ptype == MM_PORT_TYPE_IGNORED)
-        port = base_modem_create_ignored_port (self, name);
+    if (pgroup == MM_PORT_GROUP_IGNORED)
+        port = base_modem_create_ignored_port (self, ptype, name);
     else if (g_str_equal (subsys, "net"))
         port = base_modem_create_net_port (self, name);
     else if (g_str_equal (subsys, "tty"))
@@ -405,25 +424,27 @@ base_modem_internal_grab_port (MMBaseModem         *self,
     }
 
     /* Setup consecutive ports and removal watchers in all control ports */
-    if (MM_IS_PORT_SERIAL_AT (port)) {
-        mm_obj_dbg (port, "port monitoring enabled in AT port");
-        port_monitoring = TRUE;
-    } else if (MM_IS_PORT_SERIAL_QCDM (port)) {
-        mm_obj_dbg (port, "port monitoring enabled in QCDM port");
-        port_monitoring = TRUE;
-    }
+    if (pgroup == MM_PORT_GROUP_USED) {
+        if (MM_IS_PORT_SERIAL_AT (port)) {
+            mm_obj_dbg (port, "port monitoring enabled in AT port");
+            port_monitoring = TRUE;
+        } else if (MM_IS_PORT_SERIAL_QCDM (port)) {
+            mm_obj_dbg (port, "port monitoring enabled in QCDM port");
+            port_monitoring = TRUE;
+        }
 #if defined WITH_QMI
-    else if (MM_IS_PORT_QMI (port)) {
-        mm_obj_dbg (port, "port monitoring enabled in QMI port");
-        port_monitoring = TRUE;
-    }
+        else if (MM_IS_PORT_QMI (port)) {
+            mm_obj_dbg (port, "port monitoring enabled in QMI port");
+            port_monitoring = TRUE;
+        }
 #endif
 #if defined WITH_MBIM
-    else if (MM_IS_PORT_MBIM (port)) {
-        mm_obj_dbg (port, "port monitoring enabled in MBIM port");
-        port_monitoring = TRUE;
-    }
+        else if (MM_IS_PORT_MBIM (port)) {
+            mm_obj_dbg (port, "port monitoring enabled in MBIM port");
+            port_monitoring = TRUE;
+        }
 #endif
+    }
 
     if (port_monitoring) {
         if (self->priv->max_timeouts > 0)
@@ -447,6 +468,7 @@ base_modem_internal_grab_port (MMBaseModem         *self,
     if (MM_IS_PORT_SERIAL_AT (port)) {
         mm_port_serial_at_set_response_parser (MM_PORT_SERIAL_AT (port),
                                                mm_serial_parser_v1_parse,
+                                               mm_serial_parser_v1_remove_echo,
                                                mm_serial_parser_v1_new (),
                                                mm_serial_parser_v1_destroy);
         /* Prefer plugin-provided flags to the generic ones */
@@ -489,13 +511,14 @@ base_modem_internal_grab_port (MMBaseModem         *self,
 gboolean
 mm_base_modem_grab_port (MMBaseModem         *self,
                          MMKernelDevice      *kernel_device,
+                         MMPortGroup          pgroup,
                          MMPortType           ptype,
                          MMPortSerialAtFlag   at_pflags,
                          GError             **error)
 {
     g_autoptr(GError) inner_error = NULL;
 
-    if (!base_modem_internal_grab_port (self, kernel_device, FALSE, ptype, at_pflags, &inner_error)) {
+    if (!base_modem_internal_grab_port (self, kernel_device, FALSE, pgroup, ptype, at_pflags, &inner_error)) {
         /* If the port was REQUIRED via udev tags and we failed to grab it, we will report
          * a fatal error. */
         if (mm_kernel_device_get_property_as_boolean (kernel_device, ID_MM_REQUIRED)) {
@@ -542,6 +565,7 @@ mm_base_modem_grab_link_port (MMBaseModem     *self,
     port = base_modem_internal_grab_port (self,
                                           kernel_device,
                                           TRUE,
+                                          MM_PORT_GROUP_USED,
                                           MM_PORT_TYPE_NET,
                                           MM_PORT_SERIAL_AT_FLAG_NONE,
                                           error);
@@ -730,7 +754,6 @@ typedef enum {
     STATE_OPERATION_TYPE_DISABLE,
 #if defined WITH_SUSPEND_RESUME
     STATE_OPERATION_TYPE_SYNC,
-    STATE_OPERATION_TYPE_TERSE,
 #endif
 } StateOperationType;
 
@@ -767,7 +790,7 @@ state_operation_ready (MMBaseModem  *self,
     ctx = g_task_get_task_data (task);
 
     if (ctx->operation_id >= 0) {
-        mm_base_modem_operation_unlock (self, ctx->operation_id);
+        mm_iface_op_lock_unlock (MM_IFACE_OP_LOCK (self), ctx->operation_id);
         ctx->operation_id = (gssize) -1;
     }
 
@@ -814,12 +837,12 @@ lock_before_state_operation_ready (MMBaseModem  *self,
 }
 
 static void
-state_operation (MMBaseModem                  *self,
-                 StateOperationType            operation_type,
-                 MMBaseModemOperationLock      operation_lock,
-                 MMBaseModemOperationPriority  operation_priority,
-                 GAsyncReadyCallback           callback,
-                 gpointer                      user_data)
+state_operation (MMBaseModem         *self,
+                 StateOperationType   operation_type,
+                 MMOperationLock      operation_lock,
+                 MMOperationPriority  operation_priority,
+                 GAsyncReadyCallback  callback,
+                 gpointer             user_data)
 {
     GTask                 *task;
     StateOperationContext *ctx;
@@ -856,12 +879,6 @@ state_operation (MMBaseModem                  *self,
             ctx->operation        = MM_BASE_MODEM_GET_CLASS (self)->sync;
             ctx->operation_finish = MM_BASE_MODEM_GET_CLASS (self)->sync_finish;
             break;
-        case STATE_OPERATION_TYPE_TERSE:
-            operation_description = "terse";
-            optional = TRUE;
-            ctx->operation        = MM_BASE_MODEM_GET_CLASS (self)->terse;
-            ctx->operation_finish = MM_BASE_MODEM_GET_CLASS (self)->terse_finish;
-            break;
 #endif
         default:
             g_assert_not_reached ();
@@ -877,12 +894,12 @@ state_operation (MMBaseModem                  *self,
     }
     g_assert (ctx->operation && ctx->operation_finish);
 
-    if (operation_lock == MM_BASE_MODEM_OPERATION_LOCK_ALREADY_ACQUIRED) {
+    if (operation_lock == MM_OPERATION_LOCK_ALREADY_ACQUIRED) {
         state_operation_run (task);
         return;
     }
 
-    g_assert (operation_lock == MM_BASE_MODEM_OPERATION_LOCK_REQUIRED);
+    g_assert (operation_lock == MM_OPERATION_LOCK_REQUIRED);
     mm_base_modem_operation_lock (self,
                                   operation_priority,
                                   operation_description,
@@ -904,36 +921,14 @@ mm_base_modem_sync_finish (MMBaseModem   *self,
 
 void
 mm_base_modem_sync (MMBaseModem              *self,
-                    MMBaseModemOperationLock  operation_lock,
-                    GAsyncReadyCallback       callback,
-                    gpointer                  user_data)
+                    MMOperationLock      operation_lock,
+                    GAsyncReadyCallback  callback,
+                    gpointer             user_data)
 {
     state_operation (self,
                      STATE_OPERATION_TYPE_SYNC,
                      operation_lock,
-                     MM_BASE_MODEM_OPERATION_PRIORITY_DEFAULT,
-                     callback,
-                     user_data);
-}
-
-gboolean
-mm_base_modem_terse_finish (MMBaseModem   *self,
-                            GAsyncResult  *res,
-                            GError       **error)
-{
-    return state_operation_finish (self, res, error);
-}
-
-void
-mm_base_modem_terse (MMBaseModem              *self,
-                     MMBaseModemOperationLock  operation_lock,
-                     GAsyncReadyCallback       callback,
-                     gpointer                  user_data)
-{
-    state_operation (self,
-                     STATE_OPERATION_TYPE_TERSE,
-                     operation_lock,
-                     MM_BASE_MODEM_OPERATION_PRIORITY_DEFAULT,
+                     MM_OPERATION_PRIORITY_DEFAULT,
                      callback,
                      user_data);
 }
@@ -951,11 +946,11 @@ mm_base_modem_disable_finish (MMBaseModem   *self,
 }
 
 void
-mm_base_modem_disable (MMBaseModem                  *self,
-                       MMBaseModemOperationLock      operation_lock,
-                       MMBaseModemOperationPriority  operation_priority,
-                       GAsyncReadyCallback           callback,
-                       gpointer                      user_data)
+mm_base_modem_disable (MMBaseModem         *self,
+                       MMOperationLock      operation_lock,
+                       MMOperationPriority  operation_priority,
+                       GAsyncReadyCallback  callback,
+                       gpointer             user_data)
 {
     state_operation (self,
                      STATE_OPERATION_TYPE_DISABLE,
@@ -976,15 +971,15 @@ mm_base_modem_enable_finish (MMBaseModem   *self,
 }
 
 void
-mm_base_modem_enable (MMBaseModem              *self,
-                      MMBaseModemOperationLock  operation_lock,
-                      GAsyncReadyCallback       callback,
-                      gpointer                  user_data)
+mm_base_modem_enable (MMBaseModem         *self,
+                      MMOperationLock      operation_lock,
+                      GAsyncReadyCallback  callback,
+                      gpointer             user_data)
 {
     state_operation (self,
                      STATE_OPERATION_TYPE_ENABLE,
                      operation_lock,
-                     MM_BASE_MODEM_OPERATION_PRIORITY_DEFAULT,
+                     MM_OPERATION_PRIORITY_DEFAULT,
                      callback,
                      user_data);
 }
@@ -1000,15 +995,15 @@ mm_base_modem_initialize_finish (MMBaseModem   *self,
 }
 
 void
-mm_base_modem_initialize (MMBaseModem              *self,
-                          MMBaseModemOperationLock  operation_lock,
-                          GAsyncReadyCallback       callback,
-                          gpointer                  user_data)
+mm_base_modem_initialize (MMBaseModem         *self,
+                          MMOperationLock      operation_lock,
+                          GAsyncReadyCallback  callback,
+                          gpointer             user_data)
 {
     state_operation (self,
                      STATE_OPERATION_TYPE_INITIALIZE,
                      operation_lock,
-                     MM_BASE_MODEM_OPERATION_PRIORITY_DEFAULT,
+                     MM_OPERATION_PRIORITY_DEFAULT,
                      callback,
                      user_data);
 }
@@ -1295,25 +1290,28 @@ port_info_cmp (const MMModemPortInfo *a,
     return g_strcmp0 (a->name, b->name);
 }
 
-MMModemPortInfo *
-mm_base_modem_get_port_infos (MMBaseModem *self,
-                              guint       *n_port_infos)
+static MMModemPortInfo *
+parse_port_infos (GHashTable *ports,
+                  guint      *n_port_infos,
+                  MMPortGroup pgroup_filter)
 {
     GHashTableIter  iter;
     GArray         *port_infos;
     MMPort         *port;
 
-    if (!self->priv->ports) {
+    if (!ports) {
         *n_port_infos = 0;
         return NULL;
     }
 
-    *n_port_infos = g_hash_table_size (self->priv->ports);
-    port_infos = g_array_sized_new (FALSE, FALSE, sizeof (MMModemPortInfo), *n_port_infos);
-    g_hash_table_iter_init (&iter, self->priv->ports);
+    port_infos = g_array_new (FALSE, FALSE, sizeof (MMModemPortInfo));
+    g_hash_table_iter_init (&iter, ports);
     while (g_hash_table_iter_next (&iter, NULL, (gpointer)&port)) {
         MMModemPortInfo port_info;
 
+        if (mm_port_get_port_group (port) != pgroup_filter)
+            continue;
+
         port_info.name = g_strdup (mm_port_get_device (port));
         switch (mm_port_get_port_type (port)) {
         case MM_PORT_TYPE_NET:
@@ -1352,11 +1350,25 @@ mm_base_modem_get_port_infos (MMBaseModem *self,
         g_array_append_val (port_infos, port_info);
     }
 
-    g_assert (*n_port_infos == port_infos->len);
+    *n_port_infos = port_infos->len;
     g_array_sort (port_infos, (GCompareFunc) port_info_cmp);
     return (MMModemPortInfo *) g_array_free (port_infos, FALSE);
 }
 
+MMModemPortInfo *
+mm_base_modem_get_port_infos (MMBaseModem *self,
+                              guint       *n_port_infos)
+{
+    return parse_port_infos (self->priv->ports, n_port_infos, MM_PORT_GROUP_USED);
+}
+
+MMModemPortInfo *
+mm_base_modem_get_ignored_port_infos (MMBaseModem *self,
+                                      guint       *n_port_infos)
+{
+    return parse_port_infos (self->priv->ports, n_port_infos, MM_PORT_GROUP_IGNORED);
+}
+
 static gint
 port_cmp (MMPort *a,
           MMPort *b)
@@ -1498,6 +1510,10 @@ mm_base_modem_organize_ports (MMBaseModem *self,
 
     g_hash_table_iter_init (&iter, self->priv->ports);
     while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &candidate)) {
+        /* Skip ports that should not be used */
+        if (mm_port_get_port_group (candidate) != MM_PORT_GROUP_USED)
+            continue;
+
         switch (mm_port_get_port_type (candidate)) {
 
         case MM_PORT_TYPE_AT:
@@ -1775,8 +1791,8 @@ mm_base_modem_organize_ports (MMBaseModem *self,
 /*****************************************************************************/
 /* Authorization */
 
-gboolean
-mm_base_modem_authorize_finish (MMBaseModem *self,
+static gboolean
+mm_base_modem_authorize_finish (MMIfaceAuth *auth,
                                 GAsyncResult *res,
                                 GError **error)
 {
@@ -1798,24 +1814,18 @@ authorize_ready (MMAuthProvider *authp,
     g_object_unref (task);
 }
 
-void
-mm_base_modem_authorize (MMBaseModem *self,
+static void
+mm_base_modem_authorize (MMIfaceAuth *auth,
                          GDBusMethodInvocation *invocation,
                          const gchar *authorization,
                          GAsyncReadyCallback callback,
                          gpointer user_data)
 {
-    GTask *task;
+    MMBaseModem *self = MM_BASE_MODEM (auth);
+    GTask       *task;
 
     task = g_task_new (self, self->priv->authp_cancellable, callback, user_data);
 
-    /* When running in the session bus for tests, default to always allow */
-    if (mm_context_get_test_session ()) {
-        g_task_return_boolean (task, TRUE);
-        g_object_unref (task);
-        return;
-    }
-
     mm_auth_provider_authorize (self->priv->authp,
                                 invocation,
                                 authorization,
@@ -1828,10 +1838,10 @@ mm_base_modem_authorize (MMBaseModem *self,
 /* Exclusive operation */
 
 typedef struct {
-    gssize                        id;
-    MMBaseModemOperationPriority  priority;
-    gchar                        *description;
-    GTask                        *wait_task;
+    gssize               id;
+    MMOperationPriority  priority;
+    gchar               *description;
+    GTask               *wait_task;
 } OperationInfo;
 
 static void
@@ -1844,7 +1854,7 @@ operation_info_free (OperationInfo *info)
 
 /* Exclusive operation lock */
 
-gssize
+static gssize
 mm_base_modem_operation_lock_finish (MMBaseModem   *self,
                                      GAsyncResult  *res,
                                      GError       **error)
@@ -1869,7 +1879,7 @@ base_modem_operation_run (MMBaseModem *self)
     /* Run the operation in head of the list */
     mm_obj_dbg (self, "[operation %" G_GSSIZE_FORMAT "] %s - %s: lock acquired",
                 info->id,
-                mm_base_modem_operation_priority_get_string (info->priority),
+                mm_operation_priority_get_string (info->priority),
                 info->description);
     task = g_steal_pointer (&info->wait_task);
     g_task_return_int (task, info->id);
@@ -1912,7 +1922,7 @@ abort_pending_operations (MMBaseModem *self)
         g_assert (info->wait_task);
         mm_obj_dbg (self, "[operation %" G_GSSIZE_FORMAT "] %s - %s: aborted early",
                     info->id,
-                    mm_base_modem_operation_priority_get_string (info->priority),
+                    mm_operation_priority_get_string (info->priority),
                     info->description);
 
         task = g_steal_pointer (&info->wait_task);
@@ -1925,12 +1935,12 @@ abort_pending_operations (MMBaseModem *self)
     self->priv->scheduled_operations = head;
 }
 
-void
-mm_base_modem_operation_lock (MMBaseModem                   *self,
-                              MMBaseModemOperationPriority   priority,
-                              const gchar                   *description,
-                              GAsyncReadyCallback            callback,
-                              gpointer                       user_data)
+static void
+mm_base_modem_operation_lock (MMBaseModem          *self,
+                              MMOperationPriority   priority,
+                              const gchar          *description,
+                              GAsyncReadyCallback   callback,
+                              gpointer              user_data)
 {
     GTask          *task;
     OperationInfo  *info;
@@ -1957,19 +1967,19 @@ mm_base_modem_operation_lock (MMBaseModem                   *self,
     } else
         operation_id++;
 
-    if (info->priority == MM_BASE_MODEM_OPERATION_PRIORITY_OVERRIDE) {
+    if (info->priority == MM_OPERATION_PRIORITY_OVERRIDE) {
         mm_obj_dbg (self, "[operation %" G_GSSIZE_FORMAT "] %s - %s: override requested - no new operations will be allowed",
                     info->id,
-                    mm_base_modem_operation_priority_get_string (info->priority),
+                    mm_operation_priority_get_string (info->priority),
                     info->description);
         g_assert (!self->priv->scheduled_operations_forbidden_forever);
         self->priv->scheduled_operations_forbidden_forever = TRUE;
         abort_pending_operations (self);
         self->priv->scheduled_operations = g_list_append (self->priv->scheduled_operations, info);
-    } else if (info->priority == MM_BASE_MODEM_OPERATION_PRIORITY_DEFAULT) {
+    } else if (info->priority == MM_OPERATION_PRIORITY_DEFAULT) {
         mm_obj_dbg (self, "[operation %" G_GSSIZE_FORMAT "] %s - %s: scheduled",
                     info->id,
-                    mm_base_modem_operation_priority_get_string (info->priority),
+                    mm_operation_priority_get_string (info->priority),
                     info->description);
         self->priv->scheduled_operations = g_list_append (self->priv->scheduled_operations, info);
     } else
@@ -1980,10 +1990,11 @@ mm_base_modem_operation_lock (MMBaseModem                   *self,
 
 /* Exclusive operation unlock */
 
-void
-mm_base_modem_operation_unlock (MMBaseModem *self,
-                                gssize       operation_id)
+static void
+mm_base_modem_operation_unlock (MMIfaceOpLock *_self,
+                                gssize         operation_id)
 {
+    MMBaseModem   *self = MM_BASE_MODEM (_self);
     OperationInfo *info;
 
     g_assert (self->priv->scheduled_operations);
@@ -1994,7 +2005,7 @@ mm_base_modem_operation_unlock (MMBaseModem *self,
 
     mm_obj_dbg (self, "[operation %" G_GSSIZE_FORMAT "] %s - %s: lock released",
                 info->id,
-                mm_base_modem_operation_priority_get_string (info->priority),
+                mm_operation_priority_get_string (info->priority),
                 info->description);
 
     /* Remove head list item and free its contents */
@@ -2009,9 +2020,9 @@ mm_base_modem_operation_unlock (MMBaseModem *self,
 /*****************************************************************************/
 
 typedef struct {
-    GDBusMethodInvocation        *invocation;
-    MMBaseModemOperationPriority  operation_priority;
-    gchar                        *operation_description;
+    GDBusMethodInvocation *invocation;
+    MMOperationPriority    operation_priority;
+    gchar                 *operation_description;
 } AuthorizeAndOperationLockContext;
 
 static void
@@ -2022,8 +2033,8 @@ authorize_and_operation_lock_context_free (AuthorizeAndOperationLockContext *ctx
     g_slice_free (AuthorizeAndOperationLockContext, ctx);
 }
 
-gssize
-mm_base_modem_authorize_and_operation_lock_finish (MMBaseModem   *self,
+static gssize
+mm_base_modem_authorize_and_operation_lock_finish (MMIfaceOpLock *self,
                                                    GAsyncResult  *res,
                                                    GError       **error)
 {
@@ -2047,35 +2058,35 @@ lock_after_authorize_ready (MMBaseModem  *self,
 }
 
 static void
-authorize_before_lock_ready (MMBaseModem  *self,
+authorize_before_lock_ready (MMIfaceAuth  *auth,
                              GAsyncResult *res,
                              GTask        *task)
 {
     GError                           *error = NULL;
     AuthorizeAndOperationLockContext *ctx;
 
-    if (!mm_base_modem_authorize_finish (self, res, &error)) {
+    if (!mm_iface_auth_authorize_finish (auth, res, &error)) {
         g_task_return_error (task, error);
         g_object_unref (task);
         return;
     }
 
     ctx = g_task_get_task_data (task);
-    mm_base_modem_operation_lock (self,
+    mm_base_modem_operation_lock (MM_BASE_MODEM (auth),
                                   ctx->operation_priority,
                                   ctx->operation_description,
                                   (GAsyncReadyCallback) lock_after_authorize_ready,
                                   task);
 }
 
-void
-mm_base_modem_authorize_and_operation_lock (MMBaseModem                  *self,
-                                            GDBusMethodInvocation        *invocation,
-                                            const gchar                  *authorization,
-                                            MMBaseModemOperationPriority  operation_priority,
-                                            const gchar                  *operation_description,
-                                            GAsyncReadyCallback           callback,
-                                            gpointer                      user_data)
+static void
+mm_base_modem_authorize_and_operation_lock (MMIfaceOpLock         *self,
+                                            GDBusMethodInvocation *invocation,
+                                            const gchar           *authorization,
+                                            MMOperationPriority    operation_priority,
+                                            const gchar           *operation_description,
+                                            GAsyncReadyCallback    callback,
+                                            gpointer               user_data)
 {
     GTask                            *task;
     AuthorizeAndOperationLockContext *ctx;
@@ -2088,7 +2099,7 @@ mm_base_modem_authorize_and_operation_lock (MMBaseModem                  *self,
     ctx->operation_description = g_strdup (operation_description);
     g_task_set_task_data (task, ctx, (GDestroyNotify)authorize_and_operation_lock_context_free);
 
-    mm_base_modem_authorize (self,
+    mm_iface_auth_authorize (MM_IFACE_AUTH (self),
                              invocation,
                              authorization,
                              (GAsyncReadyCallback)authorize_before_lock_ready,
@@ -2630,6 +2641,21 @@ log_object_iface_init (MMLogObjectInterface *iface)
     iface->build_id = log_object_build_id;
 }
 
+static void
+auth_iface_init (MMIfaceAuthInterface *iface)
+{
+    iface->authorize = mm_base_modem_authorize;
+    iface->authorize_finish = mm_base_modem_authorize_finish;
+}
+
+static void
+op_lock_iface_init (MMIfaceOpLockInterface *iface)
+{
+    iface->authorize_and_lock = mm_base_modem_authorize_and_operation_lock;
+    iface->authorize_and_lock_finish = mm_base_modem_authorize_and_operation_lock_finish;
+    iface->unlock = mm_base_modem_operation_unlock;
+}
+
 static void
 mm_base_modem_class_init (MMBaseModemClass *klass)
 {
@@ -2728,7 +2754,7 @@ mm_base_modem_class_init (MMBaseModemClass *klass)
     g_object_class_install_property (object_class, PROP_SUBSYSTEM_DEVICE_ID, properties[PROP_SUBSYSTEM_DEVICE_ID]);
 
     properties[PROP_CONNECTION] =
-        g_param_spec_object (MM_BASE_MODEM_CONNECTION,
+        g_param_spec_object (MM_BINDABLE_CONNECTION,
                              "Connection",
                              "GDBus connection to the system bus.",
                              G_TYPE_DBUS_CONNECTION,
