@@ -33,6 +33,7 @@
 #include "mm-bearer-mbim.h"
 #include "mm-log-object.h"
 #include "mm-context.h"
+#include "mm-bind.h"
 
 G_DEFINE_TYPE (MMBearerMbim, mm_bearer_mbim, MM_TYPE_BASE_BEARER)
 
@@ -1458,11 +1459,10 @@ load_settings_from_bearer (MMBearerMbim        *self,
     guint                     max_multiplexed_bearers;
     const gchar              *data_port_driver;
 
-    if (!mm_iface_modem_count_bearers (MM_IFACE_MODEM (ctx->modem),
-                                       MM_IFACE_MODEM_COUNT_BEARERS_FLAG_MULTIPLEXED,
-                                       &current_multiplexed_bearers,
-                                       &max_multiplexed_bearers,
-                                       error))
+    if (!mm_broadband_modem_get_active_multiplexed_bearers (MM_BROADBAND_MODEM (ctx->modem),
+                                                            &current_multiplexed_bearers,
+                                                            &max_multiplexed_bearers,
+                                                            error))
         return FALSE;
 
     /* Check multiplex support in the kernel and the device */
@@ -2020,6 +2020,7 @@ mm_bearer_mbim_new (MMBroadbandModemMbim *modem,
      * g_object_new() here */
     bearer = g_object_new (MM_TYPE_BEARER_MBIM,
                            MM_BASE_BEARER_MODEM,  modem,
+                           MM_BIND_TO, G_OBJECT (modem),
                            MM_BASE_BEARER_CONFIG, config,
                            NULL);
 
