@@ -42,6 +42,7 @@
 #include "mm-modem-helpers.h"
 #include "mm-modem-helpers-mbm.h"
 #include "mm-daemon-enums-types.h"
+#include "mm-bind.h"
 
 G_DEFINE_TYPE (MMBroadbandBearerMbm, mm_broadband_bearer_mbm, MM_TYPE_BROADBAND_BEARER)
 
@@ -871,6 +872,7 @@ mm_broadband_bearer_mbm_new (MMBroadbandModemMbm *modem,
         callback,
         user_data,
         MM_BASE_BEARER_MODEM, modem,
+        MM_BIND_TO, modem,
         MM_BASE_BEARER_CONFIG, config,
         NULL);
 }
